@@ -1,64 +1,48 @@
 /*
- * Copyright Stalwart Labs, Minter Ltd. See the COPYING
- * file at the top-level directory of this distribution.
+ * SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
  *
- * Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
- * https://www.apache.org/licenses/LICENSE-2.0> or the MIT license
- * <LICENSE-MIT or https://opensource.org/licenses/MIT>, at your
- * option. This file may not be copied, modified, or distributed
- * except according to those terms.
+ * SPDX-License-Identifier: Apache-2.0 OR MIT
  */
 
-use crate::parsers::message::MessageStream;
+use std::borrow::Cow;
 
-use super::DecodeResult;
+use crate::parsers::MessageStream;
 
-pub fn decode_base64<'x>(
-    stream: &MessageStream<'x>,
-    start_pos: usize,
-    boundary: &[u8],
-    is_word: bool,
-) -> (usize, DecodeResult) {
-    let mut success = boundary.is_empty();
+#[inline(always)]
+pub fn base64_decode(bytes: &[u8]) -> Option<Vec<u8>> {
+    base64_decode_stream(bytes.iter(), bytes.len(), u8::MAX)
+}
 
+pub fn base64_decode_stream<'x>(
+    stream: impl Iterator<Item = &'x u8>,
+    stream_len: usize,
+    stop_char: u8,
+) -> Option<Vec<u8>> {
     let mut chunk: u32 = 0;
     let mut byte_count: u8 = 0;
-    let mut match_count: usize = 0;
 
-    let mut bytes_read = 0;
+    let mut buf = Vec::with_capacity(stream_len / 4 * 3);
 
-    let mut buf = Vec::with_capacity(stream.data.len() - start_pos);
+    for &ch in stream {
+        let val = BASE64_MAP[byte_count as usize][ch as usize];
 
-    for ch in stream.data[start_pos..].iter() {
-        bytes_read += 1;
+        if val < 0x01ffffff {
+            byte_count = (byte_count + 1) & 3;
 
-        if !success {
-            if *ch == boundary[match_count] {
-                match_count += 1;
-                if match_count == boundary.len() {
-                    success = true;
-                    break;
-                } else {
-                    continue;
-                }
-            } else if match_count > 0 {
-                if *ch == boundary[0] {
-                    match_count = 1;
-                    continue;
-                } else {
-                    match_count = 0;
+            if byte_count == 1 {
+                chunk = val;
+            } else {
+                chunk |= val;
+
+                if byte_count == 0 {
+                    buf.extend_from_slice(&chunk.to_le_bytes()[0..3]);
                 }
             }
-        }
-
-        let val = BASE64_MAP[byte_count as usize][*ch as usize];
-
-        if val >= 0x01ffffff {
-            if *ch == b'=' {
-                match byte_count {
+        } else {
+            match ch {
+                b'=' => match byte_count {
                     1 | 2 => {
                         buf.push(chunk.to_le_bytes()[0]);
-
                         byte_count = 0;
                     }
                     3 => {
@@ -67,128 +51,171 @@ pub fn decode_base64<'x>(
                     }
                     0 => (),
                     _ => {
-                        success = false;
-                        break;
+                        return None;
                     }
-                }
-            } else if !ch.is_ascii_whitespace() || is_word {
-                success = false;
-                break;
-            }
-            continue;
-        }
-
-        byte_count = (byte_count + 1) & 3;
-
-        if byte_count == 1 {
-            chunk = val;
-        } else {
-            chunk |= val;
-
-            if byte_count == 0 {
-                buf.extend_from_slice(&chunk.to_le_bytes()[0..3]);
+                },
+                b' ' | b'\t' | b'\r' | b'\n' => (),
+                _ => return if ch == stop_char { buf.into() } else { None },
             }
         }
     }
 
-    (
-        if success { bytes_read } else { 0 },
-        if !buf.is_empty() {
-            buf.shrink_to_fit();
-            DecodeResult::Owned(buf)
-        } else {
-            DecodeResult::Empty
-        },
-    )
+    buf.into()
 }
 
-#[cfg(test)]
-mod tests {
-    use crate::{
-        decoders::{base64::decode_base64, DecodeResult},
-        parsers::message::MessageStream,
-    };
+impl<'x> MessageStream<'x> {
+    pub fn decode_base64_mime(&mut self, boundary: &[u8]) -> (usize, Cow<'x, [u8]>) {
+        let mut chunk: u32 = 0;
+        let mut byte_count: u8 = 0;
 
-    #[test]
-    fn decode_base64_strings() {
-        let inputs = [
-            ("VGVzdA==", "Test", "", true),
-            ("WWU=", "Ye", "", true),
-            ("QQ==", "A", "", true),
-            ("cm8=", "ro", "", true),
-            (
-                "QXJlIHlvdSBhIFNoaW1hbm8gb3IgQ2FtcGFnbm9sbyBwZXJzb24/",
-                "Are you a Shimano or Campagnolo person?",
-                "",
-                true,
-            ),
-            (
-                "PCFET0NUWVBFIGh0bWw+CjxodG1sPgo8Ym9keT4KPC9ib2R5Pgo8L2h0bWw+Cg==",
-                "<!DOCTYPE html>\n<html>\n<body>\n</body>\n</html>\n",
-                "",
-                true,
-            ),
-            (
-                "PCFET0NUWVBFIGh0bWw+CjxodG1sPg\no8Ym9ke\nT4KPC 9ib2R5Pg\n o8L2h0bWw+Cg==",
-                "<!DOCTYPE html>\n<html>\n<body>\n</body>\n</html>\n",
-                "",
-                false,
-            ),
-            (
-                "PCFET0NUWVBFIGh0bWw+CjxodG1sPg\no8Ym9ke\nT4KPC 9ib2R5Pg\n o8L2h0bWw+Cg==",
-                "",
-                "",
-                true,
-            ),
-            ("w6HDqcOtw7PDug==", "áéíóú", "", true),
-            ("====", "", "\n--boundary", true),
-            ("w6HDq!cOtw7PDug=", "", "", true),
-            ("w6 HD", "", "", true),
-            ("cmáé", "", "", true),
-            ("áé", "", "", true),
-            ("w6HDqcOtw7PDug==?=", "áéíóú", "?=", true),
-            (
-                "w\n6\nH\nD\nq\nc\nO\nt\nw\n7\n P\tD u g\n==\n--boundary",
-                "áéíóú",
-                "\n--boundary",
-                false,
-            ),
-            ("w6HDqcOtw7PDug================?=", "áéíóú", "?=", false),
-            (
-                "w6HDqcOtw7PDug==\n--\n--boundary",
-                "áéíóú",
-                "\n--boundary",
-                false,
-            ),
-        ];
+        let mut buf = Vec::with_capacity(self.remaining() / 4 * 3);
+        let mut last_ch = b'\n';
+        let mut before_last_ch = 0;
+        let mut end_pos = self.offset();
 
-        for input in inputs {
-            let str = input.0.to_string();
-            let stream = MessageStream::new(str.as_bytes());
+        self.checkpoint();
 
-            let (bytes_read, result) = decode_base64(&stream, 0, input.2.as_bytes(), input.3);
-            assert_eq!(
-                bytes_read > 0,
-                !input.1.is_empty(),
-                "Failed for '{:?}'",
-                input.0
-            );
+        while let Some(&ch) = self.next() {
+            let val = BASE64_MAP[byte_count as usize][ch as usize];
 
-            if !input.1.is_empty() {
-                let bytes = match result {
-                    DecodeResult::Owned(v) => v,
-                    _ => unreachable!(),
-                };
-                let result_str = std::str::from_utf8(&bytes).unwrap();
-                //println!("'{}' -> '{}'", input.0.escape_debug(), result_str.escape_debug());
-                assert_eq!(
-                    result_str,
-                    input.1,
-                    "Failed for '{}'",
-                    input.0.escape_debug()
-                );
+            if val < 0x01ffffff {
+                byte_count = (byte_count + 1) & 3;
+
+                if byte_count == 1 {
+                    chunk = val;
+                } else {
+                    chunk |= val;
+
+                    if byte_count == 0 {
+                        buf.extend_from_slice(&chunk.to_le_bytes()[0..3]);
+                    }
+                }
+            } else {
+                match ch {
+                    b'=' => match byte_count {
+                        1 | 2 => {
+                            buf.push(chunk.to_le_bytes()[0]);
+                            byte_count = 0;
+                        }
+                        3 => {
+                            buf.extend_from_slice(&chunk.to_le_bytes()[0..2]);
+                            byte_count = 0;
+                        }
+                        0 => (),
+                        _ => {
+                            self.restore();
+                            return (usize::MAX, b""[..].into());
+                        }
+                    },
+                    b'\n' => {
+                        end_pos = if last_ch == b'\r' {
+                            self.offset() - 2
+                        } else {
+                            self.offset() - 1
+                        }
+                    }
+                    b' ' | b'\t' | b'\r' => (),
+                    b'-' => {
+                        if last_ch == b'-' {
+                            return if !boundary.is_empty() && self.try_skip(boundary) {
+                                buf.shrink_to_fit();
+                                (
+                                    if before_last_ch == b'\n' {
+                                        end_pos
+                                    } else {
+                                        self.offset() - boundary.len() - 2
+                                    },
+                                    buf.into(),
+                                )
+                            } else {
+                                self.restore();
+                                (usize::MAX, b""[..].into())
+                            };
+                        }
+                    }
+                    _ => {
+                        self.restore();
+                        return (usize::MAX, b""[..].into());
+                    }
+                }
+            }
+
+            before_last_ch = last_ch;
+            last_ch = ch;
+        }
+
+        buf.shrink_to_fit();
+        (
+            if boundary.is_empty() {
+                self.offset()
+            } else {
+                self.restore();
+                usize::MAX
+            },
+            buf.into(),
+        )
+    }
+
+    pub fn decode_base64_word(&mut self) -> Option<Vec<u8>> {
+        let mut chunk: u32 = 0;
+        let mut byte_count: u8 = 0;
+        let mut buf = Vec::with_capacity(64);
+
+        while let Some(&ch) = self.next() {
+            match ch {
+                b'=' => {
+                    match byte_count {
+                        1 | 2 => {
+                            buf.push(chunk.to_le_bytes()[0]);
+                            byte_count = 0;
+                        }
+                        3 => {
+                            buf.extend_from_slice(&chunk.to_le_bytes()[0..2]);
+                            byte_count = 0;
+                        }
+                        0 => (),
+                        _ => {
+                            // Invalid
+                            break;
+                        }
+                    }
+                }
+                b'?' => {
+                    if let Some(b'=') = self.next() {
+                        return Some(buf);
+                    } else {
+                        break;
+                    }
+                }
+                b'\n' => {
+                    if !self.next_is_space() {
+                        break;
+                    }
+                }
+                b' ' | b'\t' | b'\r' => (),
+                _ => {
+                    let val = BASE64_MAP[byte_count as usize][ch as usize];
+
+                    if val < 0x01ffffff {
+                        byte_count = (byte_count + 1) & 3;
+
+                        if byte_count == 1 {
+                            chunk = val;
+                        } else {
+                            chunk |= val;
+
+                            if byte_count == 0 {
+                                buf.extend_from_slice(&chunk.to_le_bytes()[0..3]);
+                            }
+                        }
+                    } else {
+                        break;
+                    }
+                }
             }
         }
+
+        None
     }
 }
 
@@ -362,3 +389,93 @@ pub static BASE64_MAP: &[&[u32]] = &[
         0x01ffffff, 0x01ffffff, 0x01ffffff, 0x01ffffff,
     ],
 ];
+
+#[cfg(test)]
+mod tests {
+    use crate::parsers::MessageStream;
+
+    #[test]
+    fn decode_base64() {
+        for (encoded_str, expected_result) in [
+            ("VGVzdA==", "Test"),
+            ("WWU=", "Ye"),
+            ("QQ==", "A"),
+            ("cm8=", "ro"),
+            (
+                "QXJlIHlvdSBhIFNoaW1hbm8gb3IgQ2FtcGFnbm9sbyBwZXJzb24/",
+                "Are you a Shimano or Campagnolo person?",
+            ),
+            (
+                "PCFET0NUWVBFIGh0bWw+CjxodG1sPgo8Ym9keT4KPC9ib2R5Pgo8L2h0bWw+Cg==",
+                "<!DOCTYPE html>\n<html>\n<body>\n</body>\n</html>\n",
+            ),
+            (
+                "PCFET0NUWVBFIGh0bWw+CjxodG1sPg\no8Ym9ke\nT4KPC 9ib2R5Pg\n o8L2h0bWw+Cg==",
+                "<!DOCTYPE html>\n<html>\n<body>\n</body>\n</html>\n",
+            ),
+            ("w6HDqcOtw7PDug==", "áéíóú"),
+            ("====", ""),
+            ("w6HDq!cOtw7PDug=", ""),
+            ("w6 HD qcOt", "áéí"),
+            ("cmáé", ""),
+            ("áé", ""),
+            ("w\n6\nH\nD\nq\nc\nO\nt\nw\n7\n P\tD u g\n==", "áéíóú"),
+            ("w6HDqcOtw7PDug==", "áéíóú"),
+        ] {
+            assert_eq!(
+                super::base64_decode(encoded_str.as_bytes()).unwrap_or_default(),
+                expected_result.as_bytes(),
+                "Failed for {encoded_str:?}",
+            );
+        }
+    }
+
+    #[test]
+    fn decode_base64_mime() {
+        for (encoded_str, expected_result) in [
+            ("VGVzdA==\r\n--boundary\n", "Test"),
+            (
+                "PCFET0NUWVBFIGh0bWw+CjxodG1sPg\no8Ym9ke\nT4KPC 9ib2R5Pg\n o8L2h0bWw+Cg==\r\n--boundary--\r\n",
+                "<!DOCTYPE html>\n<html>\n<body>\n</body>\n</html>\n",
+            ),
+            ("w6HDqcOtw7PDug==\r\n--boundary \n", "áéíóú"),
+            ("w\n6\nH\nD\nq\nc\nO\nt\nw\n7\n P\tD u g\n==\r\n--boundary\n", "áéíóú"),
+            ("w6HDqcOtw7PDug==--boundary", "áéíóú"),
+            (
+                "w6HDqcOtw7PDug==\n--boundary--",
+                "áéíóú",
+            ),
+            (
+                "w\n6\nH\nD\nq\nc\nO\nt\nw\n7\n P\tD u g\n==\n--boundary",
+                "áéíóú",
+            ),
+        ] {
+            let mut s = MessageStream::new(encoded_str.as_bytes());
+            let (_, result) = s.decode_base64_mime(b"boundary");
+
+            assert_eq!(
+                result,
+                expected_result.as_bytes(),
+                "Failed for {encoded_str:?}",
+            );
+        }
+    }
+
+    #[test]
+    fn decode_base64_word() {
+        for (encoded_str, expected_result) in [
+            ("w 6 H D q c O t w 7 P D u g==  ?=", "áéíóú"),
+            ("w6HDqcOtw7PDug==?=", "áéíóú"),
+            ("w6HDqc\n  Otw7PDug==?=", "áéíóú"),
+            ("w6HDqcOtw7PDug================?=", "áéíóú"),
+            ("?=", ""),
+        ] {
+            let mut s = MessageStream::new(encoded_str.as_bytes());
+            assert_eq!(
+                s.decode_base64_word().unwrap(),
+                expected_result.as_bytes(),
+                "Failed for {encoded_str:?}",
+            );
+        }
+    }
+}
