@@ -1,12 +1,7 @@
 /*
- * Copyright Stalwart Labs, Minter Ltd. See the COPYING
- * file at the top-level directory of this distribution.
+ * SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
  *
- * Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
- * https://www.apache.org/licenses/LICENSE-2.0> or the MIT license
- * <LICENSE-MIT or https://opensource.org/licenses/MIT>, at your
- * option. This file may not be copied, modified, or distributed
- * except according to those terms.
+ * SPDX-License-Identifier: Apache-2.0 OR MIT
  */
 
 pub mod map;
@@ -14,11 +9,11 @@ pub mod multi_byte;
 pub mod single_byte;
 pub mod utf;
 
-pub type DecoderFnc<'x> = fn(&'x [u8]) -> String;
+pub type DecoderFnc = fn(&[u8]) -> String;
 
 #[cfg(test)]
 mod tests {
-    use super::map::get_charset_decoder;
+    use super::map::charset_decoder;
 
     #[test]
     fn decode_charset() {
@@ -65,7 +60,7 @@ mod tests {
             ];
 
         for input in inputs {
-            let decoder = get_charset_decoder(input.0.as_bytes())
+            let decoder = charset_decoder(input.0.as_bytes())
                 .expect(&("Failed to find decoder for ".to_owned() + input.0));
 
             assert_eq!(decoder(&input.1), input.2);
